//-----------------------------------------------------------------------
// A small TSP provider DLL test program.
//-----------------------------------------------------------------------
//
// Written by J-F. LAROCHE  and E. NOULARD
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// -----------------------------------------------------------------------
//
// Project   : TSP
// Component : DLL Provider test program
// 
// -----------------------------------------------------------------------
// 
// Purpose   : A small TSP provider DLL test program.
//             Call each DLL exported functions (used for debug purpose).
// 
// -----------------------------------------------------------------------

#include <signal.h>
#include <stdio.h>
#include "tsp_provider_dll.h"
#include "tsp_common.h"

#if defined (WIN32)
#include "tsp_time.h"
typedef unsigned long sigset_t;
#endif


#ifdef WIN32
    #define assert(exp)     ((void)0)
    #include  "getopt.h"
#else
    #include <unistd.h>
    #include <assert.h>
#endif

#if defined (WIN32)
static int sigint_received = 0;
// Intercept signal function : call once
void intrpt(int signum)
{
    // Managed the SIGINT signal
    (void) signal(SIGINT, SIG_DFL);
    // End of waiting
    sigint_received = 1;
    // Could be modified to use CreateEvent, SetEvent and WaitForSingleObject functions
}
#endif


// DLL provider test program
int main(int argc, char *argv[])
{

#if defined (WIN32)
    int32_t     err;
    double      baseFrequency = 1;
    const char  *providerName = "DLL_test_provider";
    int32_t     nbSymbols     = 2;
    const char  *symbolName1  = "Inc_Ramp";
    const char  *symbolName2  = "Dec_Ramp";
    double      symbolValue1  = -100.0;
    double      symbolValue2  = 100.0;

    // Managing the SIGINT signal
    if ( SIG_ERR == signal(SIGINT, intrpt)) {
        printf ("Error : initialisation signal intercept function\n");
        exit(1);
    }

    // Inform the user about the current state
    printf ("TSP provider DLL test starting\n");

    // Initialize the DLL provider
    err = tsp_provider_dll_initialize(providerName, baseFrequency);
    if (err < 0) printf ("Error : tsp_provider_dll_initialize : %d\n", err);

    err = tsp_provider_dll_symbol_initialize(nbSymbols);
    if (err < 0) printf ("Error : tsp_provider_dll_symbol_initialize : %d\n", err);

    err = tsp_provider_dll_symbol_add(symbolName1, TSP_TYPE_DOUBLE, 1);
    if (err < 0) printf ("Error : tsp_provider_dll_symbol_add : %d\n", err);

    err = tsp_provider_dll_symbol_add(symbolName2, TSP_TYPE_DOUBLE, 1);
    if (err < 0) printf ("Error : tsp_provider_dll_symbol_add : %d\n", err);

    err = tsp_provider_dll_symbol_finalize();
    if (err < 0) printf ("Error : tsp_provider_dll_symbol_finalize : %d\n", err);

    // Inform the user about the current state
    printf ("TSP provider DLL test running\n");

    // Wait until the intercept signal function is call
    while (!sigint_received) {
        // 1Hz simulation : wait 1s
        tsp_usleep(1000);

        // Update the simulated values
        symbolValue1 += 0.1;
        symbolValue2 -= 0.1;

        err = tsp_provider_dll_data_update(0, symbolValue1, 1);
        if (err < 0) printf ("Error : tsp_provider_dll_data_update : %d\n", err);

        err = tsp_provider_dll_data_update(1, symbolValue2, 1);
        if (err < 0) printf ("Error : tsp_provider_dll_data_update : %d\n", err);

        err = tsp_provider_dll_data_commit();
        if (err < 0) printf ("Error : tsp_provider_dll_data_commit : %d\n", err);
    }

    tsp_provider_dll_finalize();   

#else
    // Inform the user that it is a Windows application ;)
    printf ("Error : This program is designed to work on Windows only, sorry\n");

    // Wait until a SIGINT signal
    sigwait(&allsigs, &whatsig);
#endif

    // Inform the user about the current state
    printf ("TSP provider DLL test ending\n");
    return 0;
}
